#include "uwbgl_D3DLight1.h"
#include "uwbgl_D3DGraphicsSystem7.h"


struct ShaderLight {
    int type;
    bool on;
    float Ambient[4],
          Diffuse[4],
          Specular[4],
          Position[3],
          Direction[3],
          Range,
          FallOff,
          Attenuation[3],
          theta,
          phi;
};

static ShaderLight gAllShaderLights[8];
static bool initialized = false;

static void InitializeShaderLights()
{
#define FillChannel3(c) {\
        gAllShaderLights[i].##c##[0] = 0.0f; \
        gAllShaderLights[i].##c##[1] = 0.0f; \
        gAllShaderLights[i].##c##[2] = 0.0f; }

#define FillChannel4(c) {\
        FillChannel3(c)\
        gAllShaderLights[i].##c##[3] = 0.0f; }


    if (!initialized) {
        for (int i = 0; i<8; i++) {
            gAllShaderLights[i].type = 0;
            gAllShaderLights[i].on = false;
            gAllShaderLights[i].phi = 0.0f;
            gAllShaderLights[i].theta = 0.0f;
            gAllShaderLights[i].FallOff = 0.0f;
            gAllShaderLights[i].Range = 0.0f;
            FillChannel4(Ambient)
            FillChannel4(Diffuse)
            FillChannel4(Specular)
            FillChannel3(Position)
            FillChannel3(Direction)
            FillChannel3(Attenuation)
        }
        initialized = true;
    }
}

void UWBD3D_Light::SetToShaderLight() const
{
    int index = m_LightID;
    D3DLIGHTTYPE type = m_d3dLight.Type;
    if (type == D3DLIGHT_POINT)
        gAllShaderLights[index].type = 0;
    else if (type == D3DLIGHT_SPOT)
        gAllShaderLights[index].type = 2;
    else
        gAllShaderLights[index].type = 1;

#define GetColorC(c) {\
    gAllShaderLights[index].##c##[0] = Get##c##().PercentRed();\
    gAllShaderLights[index].##c##[1] = Get##c##().PercentGreen();\
    gAllShaderLights[index].##c##[2] = Get##c##().PercentBlue();\
    gAllShaderLights[index].##c##[3] = Get##c##().PercentAlpha();\
    }
#define GetPos(p) {\
    vec3 pos; \
    Get##p##(pos); \
    gAllShaderLights[index].##p##[0] = pos.x; \
    gAllShaderLights[index].##p##[1] = pos.y; \
    gAllShaderLights[index].##p##[2] = pos.z; }
   

    gAllShaderLights[index].on = LightIsSwitchedOn();
    gAllShaderLights[index].Range = m_d3dLight.Range;
    gAllShaderLights[index].FallOff = m_d3dLight.Falloff;
    gAllShaderLights[index].theta = m_d3dLight.Theta;
    gAllShaderLights[index].phi = m_d3dLight.Phi;
    GetColorC(Ambient)
    GetColorC(Diffuse)
    GetColorC(Specular)
    GetPos(Position)
    GetPos(Direction)
    gAllShaderLights[index].Attenuation[0] = m_d3dLight.Attenuation0;
    gAllShaderLights[index].Attenuation[1] = m_d3dLight.Attenuation1;
    gAllShaderLights[index].Attenuation[2] = m_d3dLight.Attenuation2;
}
    

UWBD3D_Light::UWBD3D_Light(int id)
:   m_LightID(id)
  , m_bLightIsOn(false)
  , m_bDrawLight(false)
{
  // Nice set of default values ...
  SetDiffuse(0.8f);
  SetSpecular(0.2f);
  SetAmbient(0.1f);

  SetRange(200.0f);
  SetFalloff(1.0f);
  SetAttenuation0(1.0f);
  SetAttenuation1(0.0f);
  SetAttenuation2(0.0f);
  SetTheta(UWB_PI/4.0f);		// Spot light inner cone radius
  SetPhi(UWB_PI/3.0f);			// Spot light outter cone radius

  SetPosition(0.0f, 0.0f, 0.0f);
  SetDirection(0.0f, -1.0f, 0.0f);
  InitializeShaderLights();
}

UWBD3D_Light::~UWBD3D_Light()
{
}

void UWBD3D_Light::DrawLight(UWB_DrawHelper& draw_helper) {
  if (LightIsSwitchedOn() && ShouldDrawLight() )
    DrawLightGeometry(draw_helper);
}

void UWBD3D_Light::SwitchOnLight()
{
  m_bLightIsOn = true;
}

void UWBD3D_Light::SwitchOffLight()
{
  m_bLightIsOn = false;
}

void UWBD3D_Light::SetLightToDevice() const
{
  UWBD3D_GraphicsSystem& graphics = UWBD3D_GraphicsSystem::GetSystem();
  LPDIRECT3DDEVICE9 device = graphics.GetD3DDevice();
  if(NULL == device || m_LightID == -1)
    return;

  if ( LightIsSwitchedOn() ) {
      device->SetLight(m_LightID, &m_d3dLight);
      device->LightEnable(m_LightID,TRUE);
      SetToShaderLight();
  } else {
      device->LightEnable(m_LightID,FALSE);
      gAllShaderLights[m_LightID].on = false;
  }

  ID3DXEffect *effect = graphics.GetCurrentEffect();
  effect->SetInt("numLights", m_LightID+1);
  effect->SetValue( "lights", gAllShaderLights, sizeof(ShaderLight) * 4);
}


void UWBD3D_Light::SetPosition(float x, float y, float z)
{
  m_d3dLight.Position.x = x;
  m_d3dLight.Position.y = y;
  m_d3dLight.Position.z = z;
}

void UWBD3D_Light::SetPosition(const vec3& p)
{
  m_d3dLight.Position.x = p.x;
  m_d3dLight.Position.y = p.y;
  m_d3dLight.Position.z = p.z;
}

void UWBD3D_Light::GetPosition(vec3& p) const
{
  p.x = m_d3dLight.Position.x;
  p.y = m_d3dLight.Position.y;
  p.z = m_d3dLight.Position.z;
}


void UWBD3D_Light::SetDirection(float x, float y, float z)
{
  m_d3dLight.Direction.x = x;
  m_d3dLight.Direction.y = y;
  m_d3dLight.Direction.z = z;
}

void UWBD3D_Light::SetDirection(const vec3& d)
{
  m_d3dLight.Direction.x = d.x;
  m_d3dLight.Direction.y = d.y;
  m_d3dLight.Direction.z = d.z;
}

void UWBD3D_Light::GetDirection(vec3& d) const
{
  d.x = m_d3dLight.Direction.x;
  d.y = m_d3dLight.Direction.y;
  d.z = m_d3dLight.Direction.z;
}