#include "uwbgl_D3DGraphicsSystem7.h"
#include "uwbgl_D3DMeshResource3.h"

UWBD3D_MeshResource::UWBD3D_MeshResource()
: m_Mesh( NULL )
{
  m_bounds.MakeInvalid();
}

UWBD3D_MeshResource::~UWBD3D_MeshResource() 
{
  ReleaseMemory();
}

void UWBD3D_MeshResource::ReleaseMemory()
{
  __super::ReleaseMemory();
  
  m_bounds.MakeInvalid();
  m_MeshMaterials.m_array.clear();

  int numMaterials = m_MeshTextures.Count();
  for( int i = 0; i < numMaterials; i++ )
  {
    if( m_MeshTextures[i] )
      m_MeshTextures[i]->Release();
    m_MeshTextures[i] = NULL;
  }
  m_MeshTextures.m_array.clear();

  if( m_Mesh )
  {
    m_Mesh->Release();
    m_Mesh = NULL;
  }
}


// Load the mesh (in D3D .x format) and store into the buffer of the device
bool UWBD3D_MeshResource::LoadResourceFromFile(const wchar_t* fullPathName)
{
  if( NULL == m_pDevice )
    return false;

  LPD3DXBUFFER pD3DXMtrlBuffer = NULL;;
  unsigned long numMaterials = 0;
  if( FAILED( D3DXLoadMeshFromX( fullPathName, D3DXMESH_SYSTEMMEM, 
                                 m_pDevice, NULL,
                                 &pD3DXMtrlBuffer, NULL, &numMaterials, 
                                 &m_Mesh ) ) )
  {
    m_Mesh = 0;
    ReleaseMemory();
    return false;
  }

  // We need to extract the material properties and texture names from the 
  // pD3DXMtrlBuffer
  D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();

  for( int i=0; i<(int)numMaterials; i++ )
  {
    m_MeshMaterials.Append( d3dxMaterials[i].MatD3D );

    m_MeshTextures.Append( NULL );
    if( d3dxMaterials[i].pTextureFilename )
    {
      // Create the texture if there is one
      wchar_t path[MAX_PATH];
      memset( path, 0, MAX_PATH * sizeof(wchar_t) );
      const wchar_t* ptr = wcsrchr(fullPathName, L'/');
      if (NULL == ptr)
        ptr = wcsrchr(fullPathName, L'\\');

      if ( ptr )
      {
        int i = 0;
        while (ptr != &(fullPathName[i]))
        {
          path[i] = fullPathName[i];
          i++;
        }
        path[i] = NULL;
        wcscat_s(path, MAX_PATH, L"/");
      }
      
      wchar_t texname[MAX_PATH];
      ::memset( texname, 0, MAX_PATH * sizeof(wchar_t) );
      size_t num_converted = 0;
      size_t origsize = strlen(d3dxMaterials[i].pTextureFilename) + 1;
      ::mbstowcs_s( &num_converted, texname, origsize, d3dxMaterials[i].pTextureFilename, _TRUNCATE );
      wcscat_s(path, MAX_PATH, texname);
      if( FAILED( D3DXCreateTextureFromFile( m_pDevice, path, &m_MeshTextures[i] ) ) )
      {
        m_MeshTextures[i] = NULL;
      }
    }
  }
  
  // Done with the material buffer
  pD3DXMtrlBuffer->Release();

  // now compute the bounding box of this mesh 
  LPDIRECT3DVERTEXBUFFER9 pVertexBuffer = NULL;
  void*        pVertices;
  // Lock the vertex buffer, to generate a simple bounding sphere
  m_Mesh->GetVertexBuffer( &pVertexBuffer );
  pVertexBuffer->Lock( 0, 0, &pVertices, D3DLOCK_NOSYSLOCK );

  D3DXVECTOR3 min_point;
  D3DXVECTOR3 max_point;
  D3DXComputeBoundingBox( (D3DXVECTOR3*)pVertices,
                           m_Mesh->GetNumVertices(),
                           D3DXGetFVFVertexSize(m_Mesh->GetFVF()),
                           &min_point,
                           &max_point );

  pVertexBuffer->Unlock();
  pVertexBuffer->Release();

  vec3 corner1( min_point.x, min_point.y, min_point.z );
  vec3 corner2( max_point.x, max_point.y, max_point.z );
  m_bounds.SetCorners( corner1, corner2 );
  return true;
}

void UWBD3D_MeshResource::DrawMesh( bool bUseTexture, bool bShowFlatColor, UWB_Color material_color )
{
  if( NULL == m_pDevice || NULL == m_Mesh )
    return;

  D3DMATERIAL9 default_material;
  memset(&default_material, 0, sizeof(D3DMATERIAL9));
  default_material.Emissive = D3DXCOLOR(material_color.ARGB());

  DWORD old_light_state = TRUE;
  if (bShowFlatColor)
  {
    m_pDevice->GetRenderState(D3DRS_LIGHTING, &old_light_state);
    m_pDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
  }

    ID3DXEffect *effect = UWBD3D_GraphicsSystem::GetSystem().GetCurrentEffect();
    UINT numPasses;
    effect->Begin(&numPasses, 0);
    for (int i = 0; i<numPasses; i++) {
        effect->BeginPass(i);
              // Meshes are divided into subsets, one for each material. Render them in a loop
              int count = m_MeshMaterials.Count();
              for( int i=0; i<count; i++ )
              {
                // Draw the mesh subset
                m_Mesh->DrawSubset( i );
              }
          effect->EndPass();
    }
    effect->End();

}

const UWB_BoundingBox* UWBD3D_MeshResource::GetBoundingBox() const
{
  return &m_bounds;
}

void UWBD3D_MeshResource::SetFirstSubMeshMaterial(UWB_Material &m)
{
    #define ConvertColor(type)                           \
      m_MeshMaterials[0].##type##.r = m.Get##type##R();  \
      m_MeshMaterials[0].##type##.g = m.Get##type##G();  \
      m_MeshMaterials[0].##type##.b = m.Get##type##B();  \
      m_MeshMaterials[0].##type##.a = m.Get##type##A();

  if (m_MeshMaterials.Count() > 0) {
      ConvertColor(Diffuse)
      ConvertColor(Specular)
      ConvertColor(Ambient)
      ConvertColor(Emissive)
      m_MeshMaterials[0].Power = m.GetPower();
  }
}

UWB_Material UWBD3D_MeshResource::GetFirstSubMeshMaterial()
{
    UWB_Material m;

    #define ConvertToUWB(type) {                   \
      UWB_Color c(m_MeshMaterials[0].##type##.r,   \
        m_MeshMaterials[0].##type##.g,             \
        m_MeshMaterials[0].##type##.b,             \
        m_MeshMaterials[0].##type##.a);            \
        m.Set##type##(c);                          \
      }
    
  if (m_MeshMaterials.Count() > 0) {
      m.SetPower(m_MeshMaterials[0].Power);
      ConvertToUWB(Diffuse)
      ConvertToUWB(Specular)
      ConvertToUWB(Ambient)
      ConvertToUWB(Emissive)
  }
  return m;
}