// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_Common.h"
#include "uwbgl_DrawHelper5.h"

/// Return true if the volumes intersect
/// \param box1_min, box1_max The corners of the first bounding box volume to test
/// \param box2_min, box2_max The corners of the bounding box volume to test against box1
bool UWB_IntersectBoxBox( vec3 box1_min, vec3 box1_max,
                          vec3 box2_min, vec3 box2_max);

/// pure virtual base class that all bounding volume objects need to subclass from
/// A bounding volume is a volume in 3D space that is used for intersection tests.
/// Primitive geometry is given a simple bounding volume or set of bounding volumes to
/// use for quick intersection tests. Typical volume types are boxes and spheres.
class UWB_BoundingVolume
{
public:
  enum volume_type { box, sphere};

  /// Return the type of bounding volume this class is
  /// This is a very simple type of runtime type information
  virtual volume_type GetType() const = 0;

  ///
  /// Return the center of the volume
  virtual vec3 GetCenter() const = 0;

  /// Union another volume with this volume to make this volume potentially larger
  /// \param pBV the other volume to union with this
  virtual void Add(const UWB_BoundingVolume* pBV) = 0;

  /// Test if another volume shares space with this volume
  /// \return true if an intersection exists
  virtual bool Intersects(const UWB_BoundingVolume& other) const = 0;

  /// Test if a given point is inside of this volume
  /// \return true if test_point is in the space that this volume contains
  virtual bool ContainsPoint(vec3 test_point) const = 0;
  
  /// Draw this volume with a draw helper class
  virtual void Draw(UWB_DrawHelper* pDrawHelper) const{}
};

///
/// Implementation of the UWB_BoundingVolume class as an Axis-Aligned Box
class UWB_BoundingBox : public UWB_BoundingVolume
{
public:
  /// Default constructor creates an empty volume box
  /// min and max corners are both at {0,0,0}
  UWB_BoundingBox();

  /// Create a box from two corner points
  /// \param corner1, corner2 two corners of a box. These do not have to be in any order
  ///        as far as minimum/maximum corners
  UWB_BoundingBox(vec3 corner1, vec3 corner2);

  /// Create a box from a center point and distance along each axis
  /// \param center Center of the box
  /// \param width Length along x-axis
  /// \param height Length along y-axis
  /// \param depth Length along z=axis
  UWB_BoundingBox(vec3 center, float width, float height, float depth);

  // virtual function overrides
  volume_type GetType() const {return box;}
  bool Intersects(const UWB_BoundingVolume& other) const;
  void Draw(UWB_DrawHelper* pDrawHelper) const;
  vec3 GetCenter() const;
  bool ContainsPoint(vec3 location) const;
  void Add(const UWB_BoundingVolume* pBV);

  /// Set the corners of the box
  /// \param corner1, corner2 two corners of a box. These do not have to be in any order
  ///        as far as minimum/maximum corners
  void SetCorners(vec3 corner1, vec3 corner2);

  /// Move the box to a new location in space so it's center is at a given point
  /// \param new_center the new center for this box
  void MoveCenterTo( vec3 new_center );

  ///
  /// Return the minimum corner of this box
  vec3 GetMin() const {return m_min;}
  ///
  /// Return the maximum corner of this box
  vec3 GetMax() const {return m_max;}

  ///
  /// Return the length along the x-axis that this box takes up
  float Width() const;
  ///
  /// Return the length along the y-axis that this box takes up
  float Height() const;
  ///
  /// Return the length along the z-axis that this box takes up
  float Depth() const;

  /// An invalid box has a maximum corner point with smaller values than
  /// the minimum corner point.
  /// Adjust the box so this is the case
  void MakeInvalid();
  /// Test to see if this box has a maximum corner point with smaller values than
  /// the minimum corner point
  bool IsValid() const;

  /// Union a bounding box volume with this bounding box.
  /// If this box is invalid, then this will get the same volume as the passed in box
  /// If the passed in box is invalid, the function does nothing
  void Add(const UWB_BoundingBox& box);
private:
  ///
  /// minimum corner point for this box
  vec3 m_min;
  ///
  /// maximum corner point for this box
  vec3 m_max;
};
