#include "uwbgl_Primitive6.h"

UWB_Primitive::UWB_Primitive()
: m_bVisible(true)
, m_bBlendingEnabled(false)
, m_bTexturingEnabled(false)
, m_bDrawBoundingVolume(false)
, m_bLightingEnabled(true)
, m_velocity(0,0,0)
, m_FillMode(fmSolid)
, m_ShadeMode(smGouraud)
, m_PointSize(2.0f)
, m_FlatColor(0,0,0)
, m_ShadingColor(255,255,255)
{
}

UWB_Primitive::~UWB_Primitive()
{
}

void UWB_Primitive::Draw(eLevelOfDetail lod, UWB_DrawHelper& draw_helper)
{
  // if the primitive is not visible, just return
  if(!IsVisible())
    return;

  draw_helper.SetDrawPrimitive(this);
  SetupDrawAttributes(draw_helper);
  DrawPrimitive(lod, draw_helper);

  if (IsDrawBoundingVolume())
    DrawBoundingVolume(lod, &draw_helper, UWB_Color(255, 0, 0));
}

// Base class does nothing because it has no idea of what to update
void UWB_Primitive::Update(float elapsed_seconds)
{
}

// Base class does nothing
void UWB_Primitive::MouseDownVertex(int vertexID, float x, float y)
{
}

// Base class does nothing
void UWB_Primitive::MoveTo(float x, float y)
{
}

vec3 UWB_Primitive::GetLocation() const
{
  // base class just returns origin {0,0,0}
  return vec3(0,0,0);
}

void UWB_Primitive::SetupDrawAttributes(UWB_DrawHelper& draw_helper)
{
  draw_helper.ResetAttributes();
  draw_helper.SetColor1(m_FlatColor);
  draw_helper.SetMaterial( m_Material );
  draw_helper.EnableBlending( m_bBlendingEnabled );
  draw_helper.SetTextureInfo( m_TexFileName.c_str() );
  draw_helper.EnableTexture( m_bTexturingEnabled ); 
  draw_helper.EnableLighting( m_bLightingEnabled);
}

void UWB_Primitive::SetVisible(bool on)
{
  m_bVisible = on;
}
bool UWB_Primitive::IsVisible() const
{
  return m_bVisible;
}

void UWB_Primitive::SetDrawBoundingVolume(bool on)
{
  m_bDrawBoundingVolume = on;
}
bool UWB_Primitive::IsDrawBoundingVolume() const
{
  return m_bDrawBoundingVolume;
}

void UWB_Primitive::SetVelocity(vec3 velocity)
{
  m_velocity = velocity;
}
vec3 UWB_Primitive::GetVelocity()  const
{
  return m_velocity;
}
bool UWB_Primitive::IsStationary() const
{
  // return true if the velocity is VERY close to {0,0,0}
  return ( UWB_ZERO_TOLERANCE >= fabs(m_velocity.x) && 
           UWB_ZERO_TOLERANCE >= fabs(m_velocity.y) && 
           UWB_ZERO_TOLERANCE >= fabs(m_velocity.z) );
}

void UWB_Primitive::SetFlatColor(UWB_Color color)
{
  m_FlatColor = color;
}
UWB_Color UWB_Primitive::GetFlatColor() const
{
  return m_FlatColor;
}

void UWB_Primitive::SetShadingColor(UWB_Color color)
{
  m_ShadingColor = color;
}
UWB_Color UWB_Primitive::GetShadingColor() const
{
  return m_ShadingColor;
}

void UWB_Primitive::SetFillMode(eFillMode mode)
{
  m_FillMode = mode;
}
eFillMode UWB_Primitive::GetFillMode() const
{
  return m_FillMode;
}

void UWB_Primitive::SetShadeMode(eShadeMode mode)
{
  m_ShadeMode = mode;
}
eShadeMode UWB_Primitive::GetShadeMode() const
{
  return m_ShadeMode;
}

void UWB_Primitive::SetPointSize(float sz)
{
  m_PointSize = sz;
}
float UWB_Primitive::GetPointSize() const
{
  return m_PointSize;
}

// Base class just returns NULL. This means that the object has no volume.
// No volume objects may be usefull if we implement things like sprites as graphic objects
const UWB_BoundingVolume* UWB_Primitive::GetBoundingVolume( eLevelOfDetail lod) const
{
  return NULL;
}
// very simple default collision response:
// push away from each other along the direction that aligns the two primitives
void UWB_Primitive::CollisionResponse(const UWB_Primitive* pOther, vec3 other_location)
{
  vec3 dir = GetLocation() - other_location;   // this vector is along the location of the two primitives
  dir = normalize(dir);

  float mySpeed = length(GetVelocity());

  SetVelocity(mySpeed * dir);
}


void UWB_Primitive::DrawBoundingVolume( eLevelOfDetail lod, UWB_DrawHelper* pRenderer, UWB_Color color ) const
{
  const UWB_BoundingVolume* pBounds = GetBoundingVolume(lod);
  if(pBounds && pRenderer)
  {
    pRenderer->ResetAttributes();
    pRenderer->SetColor1(color);
    pRenderer->SetShadeMode( smFlat );
    pRenderer->SetFillMode( fmWireframe );
    pBounds->Draw(pRenderer);
  }
}

void UWB_Primitive::EnableBlending( bool on )
{
  m_bBlendingEnabled = on;
}
bool UWB_Primitive::IsBlendingEnabled() const
{
  return m_bBlendingEnabled;
}

void UWB_Primitive::SetTextureFileName(const wchar_t* n)
{
  m_TexFileName = n;
}
const wchar_t* UWB_Primitive::GetTextureFileName()
{
  return m_TexFileName.c_str();
}

void UWB_Primitive::EnableTexturing(bool on)
{
  m_bTexturingEnabled = on;
}
bool UWB_Primitive::IsTexturingEnabled() const 
{
  return m_bTexturingEnabled;
}

UWB_Material UWB_Primitive::GetMaterial() const 
{ 
  return m_Material; 
}

void UWB_Primitive::SetMaterial(const UWB_Material &m)
{
  m_Material = m;
}



void UWB_Primitive::EnableLighting(bool on)
{
  m_bLightingEnabled = on;
}
bool UWB_Primitive::IsLightingEnabled() const 
{
  return m_bLightingEnabled;
}